package com.emato.eoss.util.sign;

import java.nio.charset.StandardCharsets;
import java.security.*;
import java.security.interfaces.DSAPrivateKey;
import java.security.interfaces.DSAPublicKey;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;


/**
 * DSA 签名
 * DSA 主要用作签名算法
 *
 * @author Scott Chen
 * @date 2017/2/27
 */
public class Dsa {

    public static final String DSA_KEY_ALGORITHM = "DSA";
    public static final String SHA1_DSA_SIGN_ALGORITHMS = "SHA1withDSA";


    private static final String DSA_PUBLIC_KEY = "DSAPublicKey";
    private static final String DSA_PRIVATE_KEY = "DSAPrivateKey";

    /**
     * DSA密钥长度默认为1024位，密钥长度必须是64的整数倍，范围在512~1024之间
     */
    private static final int KEY_SIZE = 1024;


    // ------------------------------ 生成公钥私钥字符串 ------------------------------

    /**
     * 初始化公钥私钥 KeyPair
     *
     * @return
     * @throws Exception
     */
    public static Map<String, Object> initKey() {
        KeyPairGenerator keyPairGen = null;
        try {
            keyPairGen = KeyPairGenerator.getInstance(DSA_KEY_ALGORITHM);
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }

        keyPairGen.initialize(KEY_SIZE);
        KeyPair keyPair = keyPairGen.genKeyPair();

        //获取公私钥对象
        DSAPublicKey publicKey = (DSAPublicKey) keyPair.getPublic();
        DSAPrivateKey privateKey = (DSAPrivateKey) keyPair.getPrivate();

        Map<String, Object> keyMap = new HashMap<>();
        keyMap.put(DSA_PUBLIC_KEY, publicKey);
        keyMap.put(DSA_PRIVATE_KEY, privateKey);

        return keyMap;
    }


    /**
     * 从 KeyPair 中获取公钥字符串
     *
     * @param keyMap
     * @return
     */
    public static String getPublicKeyStr(Map<String, Object> keyMap) {
        Key key = (Key) keyMap.get(DSA_PUBLIC_KEY);
        return Base64.getEncoder().encodeToString(key.getEncoded());
    }

    /**
     * 从 KeyPair 中获取私钥字符串
     *
     * @param keyMap
     * @return
     */
    public static String getPrivateKeyStr(Map<String, Object> keyMap) {
        Key key = (Key) keyMap.get(DSA_PRIVATE_KEY);
        return Base64.getEncoder().encodeToString(key.getEncoded());
    }


    // ------------------------------ 生成公钥私钥 ------------------------------

    /**
     * 生成公钥
     *
     * @param base64PublicKeyStr 密钥字符串（经过base64编码）
     * @return
     * @throws Exception
     */
    public static PublicKey generatePublicKey(String base64PublicKeyStr) throws Exception {
        return generatePublicKey(Base64.getDecoder().decode(base64PublicKeyStr));
    }


    /**
     * 生成公钥
     *
     * @param base64PublicKeyByte 密钥字节码
     * @return
     * @throws Exception
     */
    public static PublicKey generatePublicKey(byte[] base64PublicKeyByte) throws Exception {
        X509EncodedKeySpec keySpec = new X509EncodedKeySpec(base64PublicKeyByte);
        KeyFactory keyFactory = KeyFactory.getInstance(DSA_KEY_ALGORITHM);
        PublicKey publicKey = keyFactory.generatePublic(keySpec);

        return publicKey;
    }


    /**
     * 生成私钥
     *
     * @param base64PrivateKeyStr 密钥字符串（经过base64编码）
     * @return
     * @throws Exception
     */
    public static PrivateKey generatePrivateKey(String base64PrivateKeyStr) throws Exception {
        return generatePrivateKey(Base64.getDecoder().decode(base64PrivateKeyStr));
    }

    /**
     * 生成私钥
     *
     * @param base64PrivateKeyByte 密钥字节码
     * @return
     * @throws Exception
     */
    public static PrivateKey generatePrivateKey(byte[] base64PrivateKeyByte) throws Exception {
        PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(base64PrivateKeyByte);
        KeyFactory keyFactory = KeyFactory.getInstance(DSA_KEY_ALGORITHM);
        PrivateKey privateKey = keyFactory.generatePrivate(keySpec);

        return privateKey;
    }



    // ------------------------------ 加签验签 ------------------------------

    /**
     * RSA 签名
     * 私钥签名
     *
     * @param plainStr 待签名字符串
     * @param base64PrivateKeyStr
     * @param inputCharset
     * @return 签名值
     */
    public static String sign(String plainStr, String base64PrivateKeyStr, String inputCharset){
        try
        {
            inputCharset = (inputCharset == null || "".equals(inputCharset)) ? StandardCharsets.UTF_8.toString() : inputCharset;
            return sign(plainStr.getBytes(inputCharset), Base64.getDecoder().decode(base64PrivateKeyStr));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }


    /**
     * RSA 签名
     * 私钥签名
     *
     * @param plainByte 待签名字节码
     * @param base64PrivateKeyByte 私钥
     * @return 签名值
     */
    public static String sign(byte[] plainByte, byte[] base64PrivateKeyByte) {
        try
        {
            PrivateKey priKey = generatePrivateKey(base64PrivateKeyByte);

            Signature signature = Signature.getInstance(SHA1_DSA_SIGN_ALGORITHMS);

            signature.initSign(priKey);
            signature.update(plainByte);

            byte[] signed = signature.sign();

            return Base64.getEncoder().encodeToString(signed);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }


    /**
     * RSA 签名
     * 私钥签名
     *
     * @param plainStr 待签名字符串
     * @param base64SignStr
     * @param base64PublicKeyStr
     * @param inputCharset
     * @return 签名值
     */
    public static boolean verify(String plainStr, String base64SignStr, String base64PublicKeyStr, String inputCharset){
        try
        {
            inputCharset = (inputCharset == null || "".equals(inputCharset)) ? StandardCharsets.UTF_8.toString() : inputCharset;
            return verify(
                    plainStr.getBytes(inputCharset),
                    Base64.getDecoder().decode(base64SignStr),
                    Base64.getDecoder().decode(base64PublicKeyStr)
            );
        } catch (Exception e) {
            e.printStackTrace();
        }

        return false;
    }

    /**
     * RSA 验签
     * 公钥验签
     * @param plainByte 待签名字节码
     * @param base64SignByte 签名值
     * @param base64PublicKeyByte 公钥
     * @return 布尔值
     */
    public static boolean verify(byte[] plainByte, byte[] base64SignByte, byte[] base64PublicKeyByte) {
        try
        {
            PublicKey pubKey = generatePublicKey(base64PublicKeyByte);

            Signature signature = Signature.getInstance(SHA1_DSA_SIGN_ALGORITHMS);

            signature.initVerify(pubKey);
            signature.update(plainByte);

            return signature.verify(base64SignByte);

        } catch (Exception e) {
            e.printStackTrace();
        }

        return false;
    }


}
